"""
This file (test_email.py) contains the unit tests for
functions defined in email.py which is used for sending
user email confirmation
"""
import pytest
from flask_mailman import EmailMessage

from modules.box__default.auth.email import _send_email_helper
from modules.box__default.auth.email import send_async_email
from modules.box__default.auth.models import User


@pytest.mark.parametrize(
    "email_config",
    [("MAIL_DEFAULT_SENDER", "remove"), ("MAIL_DEFAULT_SENDER", None)],
    indirect=True,
)
def test_send_email_with_no_default_sender(capfd, email_config):
    user = User.create(email="test@gmail.com", password="pass")
    token = "sometoken"
    template = "auth/emails/activate_user"
    subject = "Please confirm your email"
    context = {"token": token, "user": user}
    send_async_email(user.email, subject, template, **context)
    captured = capfd.readouterr()

    assert "Shopyo Error: MAIL_DEFAULT_SENDER not configured" in captured.out


@pytest.mark.parametrize(
    "email_config",
    [
        ("MAIL_USERNAME", "remove"),
        ("MAIL_PASSWORD", "remove"),
        ("MAIL_USERNAME", None),
        ("MAIL_PASSWORD", None),
    ],
    indirect=True,
)
def test_send_email_with_no_username_or_password_set(capfd, email_config):
    user = User.create(email="test@gmail.com", password="pass")
    token = "sometoken"
    template = "auth/emails/activate_user"
    subject = "Please confirm your email"
    context = {"token": token, "user": user}
    thread = send_async_email(user.email, subject, template, **context)
    thread.join()
    captured = capfd.readouterr()

    assert (
        "Shopyo Error: MAIL_USERNAME, and/or MAIL_PASSWORD not configured"
        in captured.out
    )


def test_send_email_using_template_on_valid_credentials(capfd):
    user = User.create(email="to@gmail.com", password="pass")
    token = "sometoken"
    template = "auth/emails/activate_user"
    subject = "Please confirm your email"
    from_email = "from@gmail.com"
    context = {"token": token, "user": user}
    thread = send_async_email(
        user.email, subject, template, from_email=from_email, **context
    )
    thread.join()
    captured = capfd.readouterr()

    assert "Please confirm your email" in captured.out
    assert "sometoken" in captured.out
    assert "to@gmail.com" in captured.out
    assert "from@gmail.com" in captured.out
    assert "Welcome to Shopyo" in captured.out
    assert "To confirm your account please click on" in captured.out
    assert "The Shopyo Team" in captured.out


def test_send_using_helper_function(test_client, flask_app, capfd):
    msg = EmailMessage(
        subject="subject of email",
        body="body of email",
        to=["to@test.com"],
        from_email="from@test.com",
    )
    _send_email_helper(flask_app, msg)
    captured = capfd.readouterr()

    assert "to@test.com" in captured.out
    assert "from@test.com" in captured.out
    assert "subject of email" in captured.out
    assert "body of email" in captured.out



import os

base_path = os.path.dirname(os.path.abspath(__file__))


class Config:
    """Parent configuration class."""

    DEBUG = False
    SQLALCHEMY_DATABASE_URI = "sqlite:///shopyo.db"
    SQLALCHEMY_TRACK_MODIFICATIONS = False
    SECRET_KEY = os.urandom(24)
    BASE_DIR = base_path
    STATIC = os.path.join(BASE_DIR, "static")
    UPLOADED_PATH_IMAGE = os.path.join(STATIC, "uploads", "images")
    UPLOADED_PATH_THUM = os.path.join(STATIC, "uploads", "thumbs")

    PASSWORD_SALT = "abcdefghi"
    # SQLALCHEMY_DATABASE_URI = "mysql+pymysql://{username}:{password}@{server_name}/{db_name}".format(
    #     username='shopyo_user',
    #     password='pass',
    #     server_name='localhost',
    #     db_name='shopyodb'
    # )


class DevelopmentConfig(Config):
    """Configurations for development"""

    ENV = "development"
    DEBUG = True
    LOGIN_DISABLED = False
    # control email confirmation for user registration
    EMAIL_CONFIRMATION_DISABLED = False
    # flask-mailman configs
    MAIL_SERVER = "smtp.googlemail.com"
    MAIL_PORT = 465
    MAIL_USE_TLS = False
    MAIL_USE_SSL = True
    MAIL_USERNAME = os.environ.get("MAIL_USERNAME")
    MAIL_PASSWORD = os.environ.get("MAIL_PASSWORD")
    MAIL_DEFAULT_SENDER = os.environ.get("MAIL_DEFAULT_SENDER")


class TestingConfig(Config):
    """Configurations for testsing"""

    ENV = "testing"
    TESTING = True
    SQLALCHEMY_DATABASE_URI = "sqlite:///testing.db"
    DEBUG = True
    LIVESERVER_PORT = 8943
    LIVESERVER_TIMEOUT = 10
    SERVER_NAME = "localhost.com"
    BCRYPT_LOG_ROUNDS = 4
    WTF_CSRF_ENABLED = False
    # control email confirmation for user registration
    EMAIL_CONFIRMATION_DISABLED = False
    # flask-mailman configs
    MAIL_BACKEND = "console"
    MAIL_USERNAME = "shopyofrom@test.com"
    MAIL_PASSWORD = "pass"
    MAIL_DEFAULT_SENDER = "shopyofrom@test.com"


app_config = {
    "development": DevelopmentConfig,
    "production": Config,
    "testing": TestingConfig,
}



from shopyoapi.init import db


class People(db.Model):
    __tablename__ = "people"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(100))
    phone = db.Column(db.Integer)
    mobile = db.Column(db.Integer)
    email = db.Column(db.String(100))
    facebook = db.Column(db.String(128))
    twitter = db.Column(db.String(128))
    linkedin = db.Column(db.String(128))
    age = db.Column(db.Integer)
    birthday = db.Column(db.String(100))
    notes = db.Column(db.String(100))
    is_manufacturer = db.Column(db.Boolean)
    manufacturer_name = db.Column(db.String(100))
    manufacturer_phone = db.Column(db.Integer)
    manufacturer_address = db.Column(db.String(200))



from datetime import datetime

from shopyoapi.init import db
from shopyoapi.models import PkModel


class Announcement(PkModel):

    __tablename__ = "announcements"
    created_date = db.Column(db.DateTime, default=datetime.now())
    title = db.Column(db.String(100))
    content = db.Column(db.String(1024))



from datetime import datetime

from shopyoapi.init import db


class Page(db.Model):

    __tablename__ = "pages"
    id = db.Column(db.Integer, primary_key=True)
    created_date = db.Column(db.DateTime, default=datetime.now())
    title = db.Column(db.String(100))
    slug = db.Column(db.String(100))
    content = db.Column(db.String(1024))

    def insert(self):
        db.session.add(self)
        db.session.commit()

    def update(self):
        db.session.commit()

    def delete(self):
        db.session.delete(self)
        db.session.commit()



import subprocess
import click

from app import app
from shopyoapi.cmd import clean
from shopyoapi.cmd import create_box
from shopyoapi.cmd import create_module
from shopyoapi.cmd import create_module_in_box
from shopyoapi.cmd import initialise
from shopyoapi.cmd import collectstatic
from shopyoapi.database import autoload_models
from shopyoapi.info import printinfo


def runserver():
    app.run(host="0.0.0.0", debug=False)


@click.command()
@click.argument("args", nargs=-1)
def process(args):
    printinfo()
    if args[0] == "initialise" or args[0] == "initialize":
        clean(app)
        autoload_models()
        initialise()
    elif args[0] == "clean":
        clean(app)
    elif args[0] == "runserver":
        runserver()
    elif args[0] == "rundebug":
        app.run(debug=True, host="0.0.0.0")
        try:
            if args[1]:
                app.run(debug=True, host="0.0.0.0", port=int(args[1]))
        except IndexError as e:
            raise e
    elif args[0] == "collectstatic":
        if len(args) == 1:
            collectstatic()
        elif len(args) == 2:
            collectstatic(target_module=args[1])
    elif args[0] == "test":
        print("test ok")
    elif args[0] == "startapp" and args[1]:
        create_module(args[1])
    elif args[0] == "startbox" and args[1]:
        create_box(args[1])
    elif args[0] == "startsubapp" and args[1] and args[3]:
        if args[2].lower() == "in":
            create_module_in_box(args[1], args[3])
    elif args[0] == "db":
        try:
            autoload_models()
            if args[1] == "migrate":
                subprocess.run(["flask", "db", "migrate"])
            elif args[1] == "upgrade":
                subprocess.run(["flask", "db", "upgrade"])
            elif args[1] == "init":
                subprocess.run(["flask", "db", "init"])
        except IndexError as e:
            print("db requires more options")
            raise e
    elif args[0] == "createconfig":
        try:
            if not os.path.exists("config.py"):
                trycopy("config_demo.py", "config.py")
            else:
                print("config.py exists")
            if not os.path.exists("config.json"):
                trycopy("config_demo.json", "config.json")
            else:
                print("config.json exists")
        except PermissionError as e:
            print(
                "Cannot continue, permission error"
                "initialising config.py and config.json, "
                "copy and rename them yourself!"
            )
            raise e
    else:
        print("Command not recognised")


if __name__ == "__main__":
    process()



"""
This file (test_admin.py) contains the functional tests for
the `admin` blueprint.

These tests use GETs and POSTs to different endpoints to check
for the proper behavior of the `admin` blueprint.
"""
import os
import json
import pytest
from flask import request
from flask import url_for
from modules.box__default.auth.models import Role
from modules.box__default.auth.models import User
from modules.box__default.auth.models import role_user_bridge


dirpath = os.path.dirname(os.path.abspath(__file__))
module_path = os.path.dirname(dirpath)

module_info = None

with open(os.path.join(module_path, "info.json")) as f:
    module_info = json.load(f)


class TestAdminInvalidAccess:
    """
    Test all admin routes for correct user authentication
    """

    routes_get = [
        "/",
        "/add",
        "/delete/<id>",
        "/edit/<id>",
        "/roles",
        "/roles/<role_id>/delete",
    ]

    routes_post = ["/update", "/roles/update", "/roles/add", "/add"]

    @pytest.mark.parametrize("route", routes_get)
    def test_redirect_if_not_logged_in_get(self, test_client, route, auth):
        auth.logout()
        response = test_client.get(
            f"{module_info['url_prefix']}{route}", follow_redirects=True
        )

        assert response.status_code == 200
        assert request.path == url_for("auth.login")

    @pytest.mark.parametrize("route", routes_post)
    def test_redirect_if_not_logged_in_post(self, test_client, route, auth):
        auth.logout()
        response = test_client.post(
            f"{module_info['url_prefix']}{route}", follow_redirects=True
        )

        assert response.status_code == 200
        assert request.path == url_for("auth.login")

    @pytest.mark.usefixtures("login_non_admin_user")
    @pytest.mark.parametrize("route", routes_get)
    def test_no_admin_access_if_not_admin_get(self, test_client, route):
        response = test_client.get(
            f"{module_info['url_prefix']}{route}", follow_redirects=True
        )

        assert response.status_code == 200
        assert request.path == url_for("dashboard.index")
        assert b"You need to be an admin to view this page" in response.data

    @pytest.mark.usefixtures("login_non_admin_user")
    @pytest.mark.parametrize("route", routes_post)
    def test_no_admin_access_if_not_admin_post(self, test_client, route):
        response = test_client.post(
            f"{module_info['url_prefix']}{route}", follow_redirects=True
        )

        assert response.status_code == 200
        assert request.path == url_for("dashboard.index")
        assert b"You need to be an admin to view this page" in response.data


@pytest.mark.usefixtures("login_admin_user")
class TestAdminEndpoints:
    """
    Test all admin api post and get requests
    """

    def test_admin_user_list_get(self, test_client):
        response = test_client.get(f"{module_info['url_prefix']}/")

        assert response.status_code == 200
        assert b"Admin" in response.data
        assert b"id" in response.data
        assert b"Email" in response.data
        assert b"Password" in response.data
        assert b"Roles" in response.data

    def test_admin_add_get(self, test_client):
        response = test_client.get(f"{module_info['url_prefix']}/add")

        assert response.status_code == 200
        assert b"Email" in response.data
        assert b"Password" in response.data
        assert b"First Name" in response.data
        assert b"Last Name" in response.data
        assert b"Admin User" in response.data

    def test_admin_add_unique_user_post(self, test_client):
        role1 = Role.create(name="test1-role")
        role2 = Role.create(name="test2-role")
        data = {
            "email": "test@gmail.com",
            "password": "pass",
            "first_name": "Test",
            "last_name": "User",
            "is_admin": "",
            f"role_{role1.id}": "",
            f"role_{role2.id}": "",
        }

        test_client.post(
            f"{module_info['url_prefix']}/add",
            data=data,
            follow_redirects=True,
        )
        test_user = User.query.filter(User.email == "test@gmail.com").scalar()

        assert test_user is not None
        assert test_user.first_name == "Test"
        assert test_user.last_name == "User"
        assert test_user.is_admin is False
        assert test_user.roles is not None
        assert len(test_user.roles) == 2
        assert role1.users[0].email == "test@gmail.com"
        assert role2.users[0].email == "test@gmail.com"

    def test_admin_add_existing_user_post(self, test_client):
        User.create(email="test@gmail.com", password="pass")
        data = {
            "email": "test@gmail.com",
            "password": "pass",
            "first_name": "Test",
            "last_name": "User",
            "is_admin": "",
        }

        response = test_client.post(
            f"{module_info['url_prefix']}/add",
            data=data,
            follow_redirects=True,
        )
        test_users = User.query.filter(User.email == "test@gmail.com").count()

        assert response.status_code == 200
        assert b"User with same email already exists" in response.data
        assert test_users == 1

    def test_admin_delete_existing_user_get(self, test_client):
        user = User(email="test@gmail.com", password="pass")
        role1 = Role(name="test1-role")
        role2 = Role(name="test2-role")
        user.roles = [role1, role2]
        user.save()

        response = test_client.get(
            f"{module_info['url_prefix']}/delete/{user.id}",
            follow_redirects=True,
        )
        test_user = User.query.filter(User.email == user.email).scalar()
        test_roles = Role.query.count()
        user_role = (
            User.query.join(role_user_bridge)
            .join(Role)
            .filter(User.id == user.id)
            .scalar()
        )

        assert response.status_code == 200
        assert test_user is None
        assert user_role is None
        assert test_roles == 2

    def test_admin_delete_nonexisting_user_get(self, test_client):
        response = test_client.get(
            f"{module_info['url_prefix']}/delete/some_id",
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert b"Unable to delete. Invalid user id" in response.data

    def test_admin_edit_existing_user_get(self, test_client):
        user = User.create(email="test@gmail.com", password="pass")

        response = test_client.get(
            f"{module_info['url_prefix']}/edit/{user.id}",
        )

        assert response.status_code == 200
        assert b"test@gmail.com" in response.data
        assert b"Edit User" in response.data

    def test_admin_edit_nonexisting_user_get(self, test_client):
        response = test_client.get(
            f"{module_info['url_prefix']}/edit/some-id", follow_redirects=True
        )

        assert response.status_code == 200
        assert b"Invalid user id" in response.data
        assert request.path == f"{module_info['url_prefix']}/"

    def test_admin_update_user_adding_new_roles_to_user(self, test_client):
        user = User.create(email="foo@gmail.com", password="pass")
        role1 = Role.create(name="test1-role")
        role2 = Role.create(name="test2-role")
        data = {
            "id": str(user.id),
            "email": "bar@gmail.com",
            "password": "newpass",
            "first_name": "Test",
            "last_name": "User",
            "is_admin": "",
            f"role_{role1.id}": "",
            f"role_{role2.id}": "",
        }

        response = test_client.post(
            f"{module_info['url_prefix']}/update",
            data=data,
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert user.email == "bar@gmail.com"
        assert user.check_password("newpass")
        assert user.first_name == "Test"
        assert user.last_name == "User"
        assert len(user.roles) == 2
        assert role1.users[0].email == "bar@gmail.com"
        assert role2.users[0].email == "bar@gmail.com"

    def test_admin_update_user_remove_old_roles_from_user(self, test_client):
        user = User(email="foo@gmail.com", password="pass", is_admin=True)
        user.is_admin = True
        role1 = Role(name="test1-role")
        role2 = Role(name="test2-role")
        user.roles = [role1, role2]
        user.save()
        data = {
            "id": str(user.id),
            "email": "bar@gmail.com",
            "first_name": "Test",
            "last_name": "User",
            "password": "   ",
            "is_admin": None,
        }

        response = test_client.post(
            f"{module_info['url_prefix']}/update",
            data=data,
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert user.email == "bar@gmail.com"
        assert user.check_password("pass")
        assert len(user.roles) == 0
        assert len(role1.users) == 0
        assert len(role2.users) == 0

    def test_admin_roles_get(self, test_client):
        response = test_client.get(f"{module_info['url_prefix']}/roles")

        assert response.status_code == 200
        assert b"Roles" in response.data

    def test_admin_roles_add_nonexisting_role_post(self, test_client):
        response = test_client.post(
            f"{module_info['url_prefix']}/roles/add",
            data=dict(name="new-role"),
            follow_redirects=True,
        )

        role = Role.query.filter(Role.name == "new-role").scalar()
        role_count = Role.query.count()

        assert response.status_code == 200
        assert role is not None
        assert role_count == 1

    def test_admin_roles_add_existing_role_post(self, test_client):
        Role.create(name="new-role")

        response = test_client.post(
            f"{module_info['url_prefix']}/roles/add",
            data=dict(name="new-role"),
            follow_redirects=True,
        )
        role_count = Role.query.count()
        role = Role.query.filter(Role.name == "new-role").scalar()

        assert response.status_code == 200
        assert b"Role already exists" in response.data
        assert role is not None
        assert role_count == 1

    def test_admin_roles_delete_nonexisting_role_get(self, test_client):
        response = test_client.get(
            f"{module_info['url_prefix']}/roles/some-id/delete",
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert request.path == f"{module_info['url_prefix']}/roles"
        assert b"Unable to delete. Invalid role id" in response.data

    def test_admin_roles_delete_existing_role_get(self, test_client):
        role1 = Role.create(name="new-role1")
        role2 = Role.create(name="new-role2")

        response = test_client.get(
            f"{module_info['url_prefix']}/roles/{role1.id}/delete",
            follow_redirects=True,
        )
        roles = Role.query.all()

        assert response.status_code == 200
        assert request.path == f"{module_info['url_prefix']}/roles"
        assert b"Role successfully deleted" in response.data
        assert roles is not None
        assert roles[0].name == role2.name
        assert len(roles) == 1

    def test_admin_roles_update_nonexisting_role_post(self, test_client):
        response = test_client.post(
            f"{module_info['url_prefix']}/roles/update",
            data=dict(role_id="some-id"),
            follow_redirects=True,
        )
        roles = Role.query.count()

        assert response.status_code == 200
        assert request.path == f"{module_info['url_prefix']}/roles"
        assert b"Unable to update. Role does not exist" in response.data
        assert roles == 0

    def test_admin_roles_update_existing_role_post(self, test_client):
        new_role = Role.create(name="new-role1")

        response = test_client.post(
            f"{module_info['url_prefix']}/roles/update",
            data=dict(role_id=new_role.id, role_name="update-role"),
            follow_redirects=True,
        )
        role = Role.query.scalar()

        assert response.status_code == 200
        assert request.path == f"{module_info['url_prefix']}/roles"
        assert b"Role successfully updated" in response.data
        assert role is not None
        assert role.name == "update-role"



# from flask import render_template
from flask import flash
from flask import request

from flask_login import login_required

from shopyoapi.forms import flash_errors
from shopyoapi.html import notify_success
from shopyoapi.module import ModuleHelp

from .forms import AnnounceForm
from .models import Announcement

mhelp = ModuleHelp(__file__, __name__)
globals()[mhelp.blueprint_str] = mhelp.blueprint
module_blueprint = globals()[mhelp.blueprint_str]


@module_blueprint.route("/")
def index():
    return mhelp.info["display_string"]


# TODO add pagination
@module_blueprint.route("/dashboard", methods=["GET"])
@login_required
def dashboard():

    context = mhelp.context()
    form = AnnounceForm()
    context.update({"form": form})
    return mhelp.render("dashboard.html", **context)


@module_blueprint.route("/add/check", methods=["GET", "POST"])
@login_required
def add_check():
    if request.method == "POST":
        form = AnnounceForm()
        if not form.validate_on_submit():
            flash_errors(form)
            return mhelp.redirect_url(
                "{}.dashboard".format(mhelp.info["module_name"])
            )
        toadd_announce = Announcement(
            content=form.content.data,
            title=form.title.data,
        )
        toadd_announce.save()
    return mhelp.redirect_url("{}.dashboard".format(mhelp.info["module_name"]))


@module_blueprint.route("/list", methods=["GET"])
@login_required
def list():
    context = mhelp.context()
    announcements = Announcement.query.all()
    context.update({"announcements": announcements})
    return mhelp.render("list.html", **context)


@module_blueprint.route("/<announce_id>/delete/check", methods=["GET"])
@login_required
def delete_check(announce_id):
    announcement = Announcement.query.get(announce_id)
    if announcement is None:
        flash(notify_warning("Wrong announcement id"))
        return mhelp.redirect_url(mhelp.method("list"))
    announcement.delete()
    return mhelp.redirect_url(mhelp.method("list"))


@module_blueprint.route("/<announce_id>/edit", methods=["GET"])
@login_required
def edit(announce_id):
    context = mhelp.context()
    announcement = Announcement.query.get(announce_id)
    if announcement is None:
        flash(notify_warning("Cannot find announcement id"))
        return mhelp.redirect_url(mhelp.method("list"))
    form = AnnounceForm(obj=announcement)

    context.update({"announcement": announcement, "form": form})
    return mhelp.render("edit.html", **context)


@module_blueprint.route("/<announce_id>/edit/check", methods=["GET", "POST"])
@login_required
def edit_check(announce_id):

    if request.method == "POST":
        announcement = Announcement.query.get(announce_id)
        form = AnnounceForm(obj=announcement)
        if not form.validate_on_submit():
            flash_errors(form)
            return mhelp.redirect_url(
                mhelp.method("edit"), announce_id=announce_id
            )
        form.populate_obj(announcement)
        announcement.update()
        flash(notify_success("Announcement updated!"))
        return mhelp.redirect_url(mhelp.method("list"))



import os

shopyoapi_path = os.path.dirname(os.path.abspath(__file__))
root_path = os.path.dirname(shopyoapi_path)
static_path = os.path.join(root_path, "static")
modules_path = os.path.join(root_path, "modules")
themes_path = os.path.join(static_path, "themes")



import os
import subprocess
import sys
from pathlib import Path

from .shopyoapi.file import trycopy
from .shopyoapi.file import trycopytree
from .shopyoapi.file import trymkdir
from .shopyoapi.file import trymkfile
from .shopyoapi.file import delete_file
from .shopyoapi.info import printinfo
from .shopyoapi.cmd_helper import tryrmtree

dirpath = Path(__file__).parent.absolute()
dirpathparent = Path(__file__).parent.parent.absolute()


def is_venv():
    return hasattr(sys, "real_prefix") or (
        hasattr(sys, "base_prefix") and sys.base_prefix != sys.prefix
    )


def conf_py_content(project_name):
    return """
# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import sys
import os

current_dir = os.path.dirname(__file__)
target_dir = os.path.abspath(os.path.join(current_dir, "../"))
sys.path.insert(0, os.path.abspath(target_dir))


# -- Project information -----------------------------------------------------

project = "{project_name}"
# copyright = ''
author = ""


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.viewcode",
    "sphinx.ext.autosectionlabel",
    "sphinx.ext.napoleon",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]


# -- Options for HTML output -------------------------------------------------
html_context = {{
    "project_links": [
        "Source Code",
        "https://github.com/<name>/<project>",
        "Issue Tracker",
        "https://github.com/<name>/<project>/issues",
    ]
}}
html_logo = "shopyo.ico"

html_sidebars = {{
    "**": ["about.html", "relations.html", "navigation.html", "searchbox.html"]
}}

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "alabaster"
html_theme_options = {{
    "github_repo": "<name>/<project>",
    "fixed_sidebar": "true",
}}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]
""".format(
        project_name=project_name
    )


def index_rst_content():
    return """
Welcome to ProjectName docs!
============================

Write project description here

.. toctree::
   :maxdepth: 9

   Docs <docs>

Indices and tables
==================

* :ref:`genindex`
* :ref:`modindex`
* :ref:`search`
"""


def docs_rst_content():
    return """
.. :tocdepth:: 5

Documentation
=============

Sphinx is included in dev_requirements.txt .
Run in main folder:

.. code:: bash

    sphinx-build -b html sphinx_source docs

to generate html pages in docs
"""

def gitignore_content():
    return '''
# Byte-compiled / optimized / DLL files
__pycache__/
*.py[cod]
*$py.class

# C extensions
*.so

# Distribution / packaging
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
*.egg-info/
.installed.cfg
*.egg
MANIFEST

# PyInstaller
#  Usually these files are written by a python script from a template
#  before PyInstaller builds the exe, so as to inject date/other infos into it.
*.manifest
*.spec

# Installer logs
pip-log.txt
pip-delete-this-directory.txt

# Unit test / coverage reports
htmlcov/
.tox/
.coverage
.coverage.*
.cache
nosetests.xml
coverage.xml
*.cover
.hypothesis/
.pytest_cache/

# Translations
*.mo
*.pot

# Django stuff:
*.log
local_settings.py
db.sqlite3

# Flask stuff:
instance/
.webassets-cache

# Scrapy stuff:
.scrapy

# Sphinx documentation
_build/

# PyBuilder
target/

# Jupyter Notebook
.ipynb_checkpoints

# pyenv
.python-version

# celery beat schedule file
celerybeat-schedule

# SageMath parsed files
*.sage.py

# Environments
.env
.venv
env/
venv/
ENV/
env.bak/
venv.bak/

# Spyder project settings
.spyderproject
.spyproject

# Rope project settings
.ropeproject

# mkdocs documentation
/site

# mypy
.mypy_cache/

# shopyo
test.db
testing.db
shopyo.db

# pycharm
.idea/

# win
*.exe
*.cs
*.bat
*.vbs

#migrations
migrations/

#vscode
.vscode
workspace.code-workspace

#uploads 
shopyo/static/uploads/products/
shopyo/static/uploads/images/
shopyo/static/uploads/thumbs/
shopyo/static/uploads/category/
shopyo/static/uploads/subcategory/

# modules in static since present in modules
shopyo/static/modules/

# ignore secrets
config.py
config.json

'''


def readme_md_content(project_name):
    return """
# {project_name}
"""


def new_project(newfoldername):
    """
    $ shopyo new blog

    creates:
        blog/
            docs/
                conf.py
                index.rst
            blog/
    """
    path = "."
    newfoldername = newfoldername.strip("/").strip("\\")
    print("creating new project {}".format(newfoldername))

    base_path = path + "/" + newfoldername
    trymkdir(base_path)
    print("created dir {} in {}".format(newfoldername, path))

    trycopytree(
        os.path.join(dirpathparent, "shopyo"),
        os.path.join(base_path, newfoldername),
    )
    delete_file(os.path.join(base_path, newfoldername, '__init__.py'))
    delete_file(os.path.join(base_path, newfoldername, '__main__.py'))
    delete_file(os.path.join(base_path, newfoldername, 'requirements.txt'))
    delete_file(os.path.join(base_path, newfoldername, 'dev_requirements.txt'))

    trycopy(
        os.path.join(dirpathparent, "shopyo", "requirements.txt"),
        os.path.join(base_path, "requirements.txt"),
    )
    with open(os.path.join(base_path, "requirements.txt"), 'a') as f:
        f.write('\nshopyo')
    trycopy(
        os.path.join(dirpathparent, "shopyo", "dev_requirements.txt"),
        os.path.join(base_path, "dev_requirements.txt"),
    )
    trymkfile(
        os.path.join(base_path, ".gitignore"),
        gitignore_content(),
    )
    trymkfile(
        os.path.join(base_path, ".nojekyll"),
        '',
    )
    trycopy(
        os.path.join(dirpathparent, "shopyo", "tox.ini"),
        os.path.join(base_path, "tox.ini"),
    )

    trymkfile(
        os.path.join(base_path, "README.md"), readme_md_content(newfoldername)
    )

    # docs
    trymkdir(os.path.join(base_path, "docs"))
    sphinx_src = os.path.join(base_path, newfoldername, "sphinx_source")
    tryrmtree(sphinx_src)
    trymkdir(sphinx_src)
    trymkfile(
        os.path.join(sphinx_src, "conf.py"),
        conf_py_content(newfoldername),
    )
    trymkdir(os.path.join(sphinx_src, "_static"))
    trymkfile(
        os.path.join(sphinx_src, "_static", "custom.css"), ""
    )
    trycopy(
        os.path.join(dirpathparent, "shopyo", "sphinx_source", "Makefile"),
        os.path.join(sphinx_src, "Makefile"),
    )
    trymkfile(
        os.path.join(sphinx_src, "index.rst"),
        index_rst_content(),
    )
    trymkfile(
        os.path.join(sphinx_src, "docs.rst"),
        docs_rst_content(),
    )
    trycopy(
        os.path.join(dirpathparent, "shopyo", "sphinx_source", "shopyo.ico"),
        os.path.join(sphinx_src, "shopyo.ico"),
    )

    print('Project', newfoldername, 'created successfully!')


def main():
    
    args = sys.argv
    if len(args) == 1:
        printinfo()
        print("No arguments supplied")
    if args[1] == "new" and len(args) == 3:
        printinfo()
        new_project(args[2])
    else:
        if not is_venv():
            print("Please use Shopyo in a virtual environment for this command")
            sys.exit()
        torun = [sys.executable, "manage.py"] + args[1:]
        subprocess.run(torun, stdout=subprocess.PIPE)


if __name__ == "__main__":
    main()
